template1 = {
    "image": "map1.png",
    "rescue_team": [
        {"position": (0, 0), "seats": 2},
        {"position": (50, 100), "seats": 2},
        {"position": (100, 200), "seats": 2},
        {"position": (200, 300), "seats": 2},
    ],
    "rescue_area": [
        {"position": (0, 0), "number_of_victims": 3, "important_score": 3},
        {"position": (0, 0), "number_of_victims": 3, "important_score": 3},
        {"position": (0, 0), "number_of_victims": 3, "important_score": 3},
        {"position": (0, 0), "number_of_victims": 3, "important_score": 3},
        {"position": (0, 0), "number_of_victims": 3, "important_score": 3},
    ],
}
